@@ -22,6 +22,15 @@ from xlsxwriter.worksheet import Worksheet
 from xlsxwriter.sharedstrings import SharedStringTable
 from xlsxwriter.format import Format
 from xlsxwriter.packager import Packager
+from .utility import xl_cell_to_rowcol
+from xlsxwriter.chart_area import ChartArea
+from xlsxwriter.chart_bar import ChartBar
+from xlsxwriter.chart_column import ChartColumn
+from xlsxwriter.chart_line import ChartLine
+from xlsxwriter.chart_pie import ChartPie
+from xlsxwriter.chart_radar import ChartRadar
+from xlsxwriter.chart_scatter import ChartScatter
+from xlsxwriter.chart_stock import ChartStock
 
 
 class Workbook(xmlwriter.XMLwriter):
@@ -149,6 +158,53 @@ class Workbook(xmlwriter.XMLwriter):
 
         return xf_format
 
+    def add_chart(self, options):
+        """
+        Create a chart object.
+
+        Args:
+            options: The chart type and subtype options.
+
+        Returns:
+            Reference to a Chart object.
+
+        """
+
+        # Type must be specified so we can create the required chart instance.
+        chart_type = options.get('type', 'None')
+        if chart_type is None:
+            warn("Chart type must be defined in add_chart()")
+            return
+
+        if chart_type == 'area':
+            chart = ChartArea(options)
+        elif chart_type == 'bar':
+            chart = ChartBar(options)
+        elif chart_type == 'column':
+            chart = ChartColumn(options)
+        elif chart_type == 'line':
+            chart = ChartLine(options)
+        elif chart_type == 'pie':
+            chart = ChartPie(options)
+        elif chart_type == 'radar':
+            chart = ChartRadar(options)
+        elif chart_type == 'scatter':
+            chart = ChartScatter(options)
+        elif chart_type == 'stock':
+            chart = ChartStock(options)
+        else:
+            warn("Unknown chart type '%s' in add_chart()" % chart_type)
+            return
+
+        # Set the embedded chart name if present.
+        if 'name' in options:
+            chart.chart_name = options['name']
+
+        chart._set_embedded_config_data()
+        self.charts.append(chart)
+
+        return chart
+
     def close(self):
         """
         Call finalisation code and close file.
@@ -302,7 +358,7 @@ class Workbook(xmlwriter.XMLwriter):
                 sheet.active = 1
 
         # Convert the SST strings data structure.
-        # self._prepare_sst_string_data()
+        self._prepare_sst_string_data()
 
         # Prepare the worksheet VML elements such as comments and buttons.
         self._prepare_vml()
@@ -314,7 +370,7 @@ class Workbook(xmlwriter.XMLwriter):
         self._prepare_drawings()
 
         # Add cached data to charts.
-        # self._add_chart_data()
+        self._add_chart_data()
 
         # Package the workbook.
         packager._add_workbook(self)
@@ -668,7 +724,7 @@ class Workbook(xmlwriter.XMLwriter):
 
             for index in range(chart_count):
                 chart_ref_id += 1
-                # sheet._prepare_chart(index, chart_ref_id, drawing_id)
+                sheet._prepare_chart(index, chart_ref_id, drawing_id)
 
             for index in range(image_count):
                 filename = sheet.images[index][2]
@@ -687,9 +743,7 @@ class Workbook(xmlwriter.XMLwriter):
 
         # Sort the workbook charts references into the order that the were
         # written from the worksheets above.
-        # chart_data =  self.charts
-        # chart_data = sort { a.id <=> b.id } chart_data
-        # self.charts = chart_data
+        self.charts = sorted(self.charts, key=lambda chart: chart.id)
 
         self.drawing_count = drawing_id
 
@@ -854,6 +908,99 @@ class Workbook(xmlwriter.XMLwriter):
                                   'color_indexed': 81, 'font_only': True})
             xf._get_xf_index()
 
+    def _add_chart_data(self):
+        # Add "cached" data to charts to provide the numCache and strCacher
+        # data for series and title/axis ranges.
+        worksheets = {}
+        seen_ranges = {}
+
+        # Map worksheet names to worksheet objects.
+        for worksheet in self.worksheets():
+            worksheets[worksheet.name] = worksheet
+
+        for chart in self.charts:
+
+            for c_range in chart.formula_ids.keys():
+                r_id = chart.formula_ids[c_range]
+
+                # Skip if the series has user defined data.
+                if chart.formula_data[r_id] is not None:
+                    if (not c_range in seen_ranges
+                            or seen_ranges[c_range] is None):
+                        data = chart.formula_data[r_id]
+                        seen_ranges[c_range] = data
+                    continue
+
+                # Check to see if the data is already cached locally.
+                if c_range in seen_ranges:
+                    chart.formula_data[r_id] = seen_ranges[c_range]
+                    continue
+
+                # Convert the range formula to a sheet name and cell range.
+                (sheetname, cells) = self._get_chart_range(c_range)
+
+                # Skip if we couldn't parse the formula.
+                if sheetname is None:
+                    continue
+
+                # Die if the name is unknown since it indicates a user error in
+                # a chart series formula.
+                if not sheetname in worksheets:
+                    warn("Unknown worksheet reference '%s' in range "
+                         "'%d' passed to add_series()" % (sheetname, c_range))
+
+                # Find the worksheet object based on the sheet name.
+                worksheet = worksheets[sheetname]
+
+                # Get the data from the worksheet table.
+                data = worksheet._get_range_data(*cells)
+
+                # TODO
+                #   # Ignore rich strings for now. Deparse later if necessary.
+                #        if token =~ m{^<r>} and token =~ m{</r>$}:
+                #            token = ''
+
+                # Add the data to the chart.
+                chart.formula_data[r_id] = data
+
+                # Store range data locally to avoid lookup if seen again.
+                seen_ranges[c_range] = data
+
+    def _get_chart_range(self, c_range):
+        # Convert a range formula such as Sheet1!$B$1:$B$5 into a sheet name
+        # and cell range such as ( 'Sheet1', 0, 1, 4, 1 ).
+
+        # Split the range formula into sheetname and cells at the last '!'.
+        # TODO. Fix this to match from right.
+        pos = c_range.find('!')
+        if pos > 0:
+            sheetname, cells = c_range.split('!')
+        else:
+            return None
+
+        # Split the cell range into 2 cells or else use single cell for both.
+        if cells.find(':') > 0:
+            (cell_1, cell_2) = cells.split(':')
+        else:
+            (cell_1, cell_2) = (cells, cells)
+
+        # Remove leading/trailing quotes and convert escaped quotes to single.
+        sheetname = sheetname.strip("'")
+        sheetname = sheetname.replace("''", "'")
+
+        (row_start, col_start) = xl_cell_to_rowcol(cell_1)
+        (row_end, col_end) = xl_cell_to_rowcol(cell_2)
+
+        # Check that we have a 1D range only.
+        if row_start != row_end and col_start != col_end:
+            return None
+
+        return (sheetname, [row_start, col_start, row_end, col_end])
+
+    def _prepare_sst_string_data(self):
+        # Convert the SST string data from a dict to a list.
+        self.str_table._sort_string_data()
+
     ###########################################################################
     #
     # XML methods.
